use crate::vm::dvm_value::DvmValue;
use crate::vm::mem::DmAllocObject;
use crate::vm::DmVirtualMachine;
use std::rc::Rc;

pub fn dm_print(args: Vec<DvmValue>, vm: &mut DmVirtualMachine) -> DvmValue {
    if args.len() != 1 {
        return DvmValue::Unit; // TODO: make exception
    }
    print!("{}", get_string(&args[0], vm));
    DvmValue::Unit
}

pub fn dm_println(args: Vec<DvmValue>, vm: &mut DmVirtualMachine) -> DvmValue {
    if args.len() != 1 {
        return DvmValue::Unit;
    }
    println!("{}", get_string(&args[0], vm));
    DvmValue::Unit
}

fn get_string(dvm_value: &DvmValue, vm: &mut DmVirtualMachine) -> String {
    match dvm_value {
        DvmValue::Byte(b) => b.to_string(),
        DvmValue::Int(i) => i.to_string(),
        DvmValue::Long(l) => l.to_string(),
        DvmValue::Double(d) => d.to_string(),
        DvmValue::Boolean(b) => b.to_string(),
        DvmValue::Pointer(alloc_object_rc) => convert_to_string(alloc_object_rc.clone(), vm),
        DvmValue::ByteArray(_) => todo!(),
        DvmValue::IntArray(_) => todo!(),
        DvmValue::LongArray(_) => todo!(),
        DvmValue::DoubleArray(_) => todo!(),
        DvmValue::BooleanArray(_) => todo!(),
        DvmValue::PointerArray(_) => todo!(),
        DvmValue::Unit => String::from("Unit"),
    }
}

fn convert_to_string(alloc_object_rc: Rc<DmAllocObject>, vm: &mut DmVirtualMachine) -> String {
    let to_string_result = vm.call_by_fqn(
        "std::Display::to_string",
        vec![DvmValue::Pointer(alloc_object_rc)],
    );
    if let DvmValue::Pointer(to_string_result_alloc_object) = to_string_result {
        todo!("Convert to_string_result_alloc_object (a String) to bytes to pass to print")
    } else {
        panic!("to_string did not return a pointer")
    }
}
